##################################################################
# The LIME-UNIX compilation flow mk file
#
#Copyright 2008 Pjotr Kourzanov
#
#This file is part of LIME.
#
#LIME is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License version 2
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#LIME is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with LIME.  If not, see <http://www.gnu.org/licenses/>.
##################################################################
include $(lib)/LIME.mk

%-app.c: %.gxf
	$e$(gxf2app) $< > $@ || $(fail)

$d:
	$e$(mkdir) -p $@

define comp-rule

$d/$1/inc $d/$1/src $d/$1/docs:
	$e$(mkdir) -p $$@

$d/$1/docs/$1.pdf: $d/$1/docs
	$eif [ -r $1.pdf ]; then $(cp) $1.pdf $$@; else $(cp) $1-gen.pdf $$@; fi

$d/$1/docs/$1.html: $d/$1/docs
	$eif [ -r $1.html ]; then $(cp) $1.html $$@; else $(sed) -e 's,-gen-hier.png,-hier.png,g' $1-gen.html > $$@; fi

$d/$1/docs/$1-hier.png: $d/$1/docs
	$eif [ -r $1-hier.png ]; then $(cp) $1-hier.png $$@; else $(cp) $1-gen-hier.png $$@; fi

$d/$1/inc/$1-shell.h: $$(graphs) $d/$1/$1.imports
	$e$(gxf2shell.h) -vPE=$1 -vIMPORTS=$$(filter %.imports,$$^) $$< > $$@ || $(fail)

$d/$1/src/$1-shell.c: $$(graphs) $d/$1/$1.imports
	$e$(gxf2shell.c) -vPE=$1 -vIMPORTS=$$(filter %.imports,$$^) $$<\
		`$(sed) -e 's,$$$$,.types,g;s,^,$d/$1/,g' $$(filter %.imports,$$^)`\
	> $$@ || $(fail)

$d/$1/src/$1.mod.c: $1.mod.c
	$e$(cp) $$< $$@

$d/$1/$1.mod.o: $1.mod.o
	$e$(cp) $$< $$@

$d/$1/makefile $d/$1/diversity: $(src)/comp-config
	$e$(sed) -s 's/$$$$(comp)/$1/g' $$< | $(unbundle) -vprefix=$d
	$e$(touch) $$@

$d/$1/$1.module: $d $d/$1/inc $d/$1/src\
		$1.module\
		$d/$1/inc/$1-shell.h $d/$1/src/$1-shell.c\
		$d/$1/$1.mod.o\
		$d/$1/src/$1.mod.c\
		$d/$1/makefile $d/$1/diversity
	$e$(cp) $1.module $$@

$d/$1/$1.imports: $1.imports
	$efor i in `$(sed) -e 's,.*,&.h,g' $$(filter %.imports,$$^)`; do\
		[ -r $$$$i ] || continue;\
		cp $$$$i $d/$1/inc;\
	done
	$efor i in `$(sed) -e 's,.*,&.types,g' $$(filter %.imports,$$^)`; do\
		[ -r $$$$i ] || continue;\
		cp $$$$i $d/$1;\
	done
	$e$(sed) 's,.*/,,g' $$< > $$@

mods+=$d/$1/$1.module
pdfs+=$d/$1/docs/$1.pdf
htmls+=$d/$1/docs/$1.html $d/$1/docs/$1-hier.png

endef

$(foreach c,$(comps),$(eval $(call comp-rule,$c)))

comps: $(mods)

comps-doc: $(pdfs) $(htmls)
