/****************************************************************
This file is part of LIME.

LIME is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

LIME is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with LIME.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************/
#include UNIX

/* In this module, we name the processing elements as
 * process1 and process2, implying that the lexographic
 * order is same as the execution order such that the
 * schedule array need not be specified.
 *
 * Note that that declaration order is irrelevant, process2
 * can be declared after process1 and vice verse to the same
 * effect...
 */
void process2(const unsigned int buf[restrict BUF_SIZE],
	      unsigned int obuf2[restrict BUF_SIZE-SPLIT_BUF_SIZE])
{
	unsigned int i;
	for(i=SPLIT_BUF_SIZE; i< BUF_SIZE; i++)
		obuf2[i-SPLIT_BUF_SIZE]=buf[i];
}

void process1(const unsigned int buf[restrict BUF_SIZE],
	      unsigned int obuf1[restrict SPLIT_BUF_SIZE])
{
	unsigned int i;
	for(i=0; i< SPLIT_BUF_SIZE; i++)
		obuf1[i]=buf[i];
}
/* No schedule array is required here, since the
 * execution of lexographic orderings of processing elements
 * coincide */

#if defined(__LIME__)
void (*schedule[])() = {
[2] =	process2,
[1] =	process1,
};
#endif
