#!/usr/bin/gawk -f
##################################################################
# This AWK script implements composition of LIME nodes (.module) 
# from raw input (.mod)
#
#Copyright 2008 Pjotr Kourzanov
#
#This file is part of LIME.
#
#LIME is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License version 2
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#LIME is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with LIME.  If not, see <http://www.gnu.org/licenses/>.
##################################################################

func handle(tag,body) { return handle_gxf(tag,handle_types(tag,body)) }

END {
    merge_types()

    for (f in files) for (i=1; i<=ncomps[f]; i++) {
	c=comps[f,i]
	compose(f,c)
	printf "<node xlink:type='resource' "\
		"id='%s' "\
		"xlink:label='%s'>\n", unquote(c), unquote(c)
	#print "<node id='" c "'>"
	if (node_type[f,c]) print space(2) "<type>" node_type[f,c] "</type>"

	for (j=1; j<=nports[f,c]; j++)
		xml_port(f,c,ports[f,c,j],2)

	for (k=1; k<=norder[f,c]; k++) {
		n=order[f,c,k]
		printf space(2) "<node xlink:type='resource' "\
				"id='%s' "\
				"xlink:label='%s'>",unquote(n),unquote(n)

		if (node_type[f,n]) print space(2) "<type>" node_type[f,n] "</type>"

		print space(2) "<ret>" node_ret[f,n] "</ret>"
		for (j=1; j<=nports[f,n]; j++) 
			xml_port(f,n,ports[f,n,j],4)
		print space(2) "</node>"
	}

    	for (k=1; k<=nedges[f]; k++) {
		split(edge_from[f,k],fr,SUBSEP)
		split(edge_to[f,k],to,SUBSEP)
		printf space(2) "<endpoint xlink:type='locator' "\
			       	"xlink:label='%s:%s' "\
			       	"xlink:href='#%s:%s:%s'/>\n",
 				unquote(fr[1]),unquote(fr[2]),
				unquote(c),unquote(fr[1]),unquote(fr[2])
		printf space(2) "<endpoint xlink:type='locator' "\
			       	"xlink:label='%s:%s' "\
			       	"xlink:href='#%s:%s:%s'/>\n",
 				unquote(to[1]),unquote(to[2]),
				unquote(c),unquote(to[1]),unquote(to[2])
		printf space(2) "<edge xlink:type='arc' "\
				"xlink:from='%s:%s:%s' "\
		                "xlink:to='%s:%s:%s' "\
				"type='%s'>\n"\
			        "<from><node id='%s'>\n"\
				      "<port id='%s'/></node></from>\n"\
			        "<to><node id='%s'>\n"\
				    "<port id='%s'/></node></to>\n"\
				"</edge>\n",
				unquote(c),unquote(fr[1]),unquote(fr[2]),
 				unquote(c),unquote(to[1]),unquote(to[2]),
 				edge_type[f,k],
				unquote(fr[1]), unquote(fr[2]),
 				unquote(to[1]), unquote(to[2])

    	}
	for (j=1; j<=nports[f,c]; j++) {
		p=ports[f,c,j]
		ns=split(port_source[f,c,p],b)
		for(k=1; k<=ns; k++) 
		{
		    split(b[k],bp,SUBSEP)

		    n=unquote(bp[1])
		    np=unquote(bp[2])

		    #print f,c,p,bp[1],bp[2],peer_node[f,n,np] >STDERR
		    #print f,c,p,n,np,init_node[f,n,np] >STDERR
		    #print f,c,p,n,np,deinit_node[f,n,np] >STDERR

		    if (peer_node[f,n,np] && edge_type[f,peer_edge[f,n,np]]!="state") continue
		    if (init_node[f,n,np] && init_dir[f,n,np]=="write") continue
		    if (deinit_node[f,n,np] && deinit_dir[f,n,np]=="read") continue

		    ref=n ":" np

		    printf "<bound-to xlink:type='arc' "\
		       "xlink:from='%s:%s' "\
		       "xlink:to='%s:%s'/>\n",
		       unquote(c),unquote(p),
		       unquote(c),ref
		}

	}

	print "</node>"
    }
}

func compose(f,c, i,j,k,n,p,seen,tf,a,t,dp)
{
	delete seen
	nports[f,c]=ninports[f,c]=noutports[f,c]=0
	for (i=1; i<=norder[f,c]; i++) 
	{	n=order[f,c,i]

		for (j=1; j<=nports[f,n]; j++) 
		{	p=ports[f,n,j]
			# the type of the port is a
			# discriminated union - expand it
			if (split(variants[port_type[f,n,p]],a,SUBSEP)==2)
			{   tf=a[1]; ti=a[2]; t=types[tf,ti]
			    # aggregator can not be of a larger domain
			    # than that of the selector
			    if (portsize(f,n,p) > nports[tf,t])
			    	fail("aggregator has more capacity that the selector!")
			    for (k=1; k<=nports[tf,t]; k++)
			    {	dp=ports[tf,t,k]
				copy_port(f,c,dp,n,p,port_const[f,n,dp],port_type[tf,t,dp],port_size[tf,t,dp])
			    }
			    # make an extra port for the tag
			    if (is_node_type(f,n,"unfold")) {
				dp=unquote(p) "_" typesels[tf,ti]
				copy_port(f,c,dp,n,p,port_const[f,n,dp],seltype[tf,ti])
				handled[f,n,"unfold"]=1
			    }
			    seen[p]=port_const[f,n,p]
			} else {
			}
			if (p in seen) {
				port_source[f,c,p]=append(port_source[f,c,p],n SUBSEP p)
				if (seen[p]!=port_const[f,n,p]) fail("port type mismatch " f " " n " " p)
				continue
			}
			copy_port(f,c,p,n,p,port_const[f,n,p])
			seen[p]=port_const[f,n,p]
		}

		inherit_types(f,c,n)
	}
}

func copy_port(f,c,dp,n,p,input,type,size) {
	# by default, take the type and the size from the source
	if (!type) type=port_type[f,n,p]
	if (!size) size=port_size[f,n,p]

	ports[f,c,++nports[f,c]]=dp
	if (input)
		inports[f,c,++ninports[f,c]]=dp
	else
		outports[f,c,++noutports[f,c]]=dp
	# inherit modifiers from the union
	port_const[f,c,dp]=port_const[f,n,p]
	port_volatile[f,c,dp]=port_volatile[f,n,p]
	port_constp[f,c,dp]=port_constp[f,n,p]
	port_volatilep[f,c,dp]=port_volatilep[f,n,p]
	port_static[f,c,dp]=port_static[f,n,p]
	port_restrict[f,c,dp]=port_restrict[f,n,p]
	# use typeinfo from the type files
	port_type[f,c,dp]=type
	port_size[f,c,dp]=size

	port_source[f,c,dp]=append(port_source[f,c,dp],n SUBSEP p)
}

func xml_port(f,c,p,l) {
	printf space(l) "<port xlink:type='resource' "\
		"id='%s' "\
		"xlink:label='%s'>\n",unquote(p),unquote(p)
	print space(l+2) "<type>" port_type[f,c,p] "</type>"
	xml_portsizes(f,c,p,l+2)
	print space(l+2) (port_const[f,c,p] ? "<const/>" : ""),
	               (port_volatile[f,c,p] ? "<volatile/>" : ""),
	               (port_constp[f,c,p] ? "<constp/>" : ""),
	               (port_volatilep[f,c,p] ? "<volatilep/>" : ""),
           	       (port_static[f,c,p] ? "<static/>" : ""),
	               (port_restrict[f,c,p] ? "<restrict/>" : "")
	print space(l) "</port>"
}

@include pragmas.awk
@include gxf-parser.awk
@include types-parser.awk
