##################################################################
# This mk file contains rules to allow specifiction of LIME nodes
# using the noweb literate programming tool and wesp tooling
#
#Copyright 2008 Pjotr Kourzanov
#
#This file is part of LIME.
#
#LIME is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License version 2
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#LIME is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with LIME.  If not, see <http://www.gnu.org/licenses/>.
##################################################################

######### literate stuff
%.c: %.nw
	$(inform)
	$e$(not) $< > $@



%.pdf: %.dot
	$(inform)
	$e$(dot) -T$(patsubst .%,%,$(suffix $@)) $< > $@

%.ps: %.dot
	$(inform)
	$e$(dot) -T$(patsubst .%,%,$(suffix $@)) $< > $@

%.png: %.dot
	$(inform)
	$e$(dot) -T$(patsubst .%,%,$(suffix $@)) $< > $@

%.defs: %.nw
	$(inform)
	$e$(nod) $< > $@



%-gen.noweb: %.c
	$(inform)
	$e(\
	    t=`file -b $<`;\
	    if [ "$${t/LaTeX/}" != "$${t}" ]; then\
		$(cat) $< | $(cast);\
	    else\
		$(cat) $< | $(cast) -vVERBATIM=1;\
	    fi;\
	) > $@

%-gen.nw: %-gen.noweb
	$(inform)
	$e$(cat) $< | $(unmarkup) > $@

%-gen-hier.dot:  %-gen.noweb
	$(inform)
	$eMARKUP=cat $(gendeps) $< > $@

%-gen.nw.tex: %-gen.noweb
	$(inform)
	$e(\
		cat $(filter %.noweb,$^);\
	) | $(indent) $(nopp) | $(autodef) | $(finduses) | $(noidx) \
	  | $(fixmarkup) | $(expand) | $(totex) -no-gen-comment > $@

%-gen.html: $(before.tex) %-gen.noweb $(after.tex)
	$(inform)
	$e(\
		$(call htmlit.sed,$*) $(filter %before.tex,$^) | $(markup);\
		$(cat) $(filter %.noweb,$^);\
		$(call htmlit.sed,$*-gen) $(filter %after.tex,$^) | $(markup);\
	) | $(indent) $(nopp) | $(autodef) | $(finduses) | $(noidx) \
	  | $(fixtex) | $(l2h) $(redir) | $(fixhtml) | $(tohtml) \
	  | $(htmltoc) > $@


%-hier.dot:  %.nw
	$(inform)
	$e$(gendeps) $< > $@

%.nw.tex: %.nw
	$(inform)
	$e$(now) -latex -index -delay $(filter %.nw,$^) > $@

%.html: $(before.tex) %.nw $(after.tex)
	$(inform)
	$e$(now) -html -filter fixtex -filter l2h -index -filter fixhtml\
		<($(call htmlit.sed,$*) $(filter %before.tex,$^))\
		$(filter %.nw,$^)\
		<($(call htmlit.sed,$*) $(filter %after.tex,$^)) $(redir) | $(htmltoc) > $@


%.tex: $(before.tex) %.nw.tex $(after.tex)
	$(inform)
	$e( $(call texit.sed,$(subst _,\\_,$(subst -gen,,$*))) $(filter %before.tex,$^);\
	  $(cat) $(filter %.nw.tex,$^);\
	  $(call texit.sed,$*) $(filter %after.tex,$^);\
	) > $@

%.pdf: %.tex
	$(inform)
	$e$(pdflatex) $< $(quiet)
	$e$(pdflatex) $< $(quiet)
	$e$(pdflatex) $< $(quiet)
	$e$(rm) $*.{toc,out,log,aux}
