##################################################################
# This mk file contains rules (no configuration or command defines)
# that drive the LIME flow
#
#Copyright 2008 Pjotr Kourzanov
#
#This file is part of LIME.
#
#LIME is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License version 2
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#LIME is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with LIME.  If not, see <http://www.gnu.org/licenses/>.
##################################################################
include $(rlib)/LIME.mk
include $(rlib)/literate.mk
include $(rlib)/gcc.mk

######### compilation stuff

%.domains: %.c
	$(inform)
	$e(\
		$(call c2dom,$<);\
		for x in $(syshdr); do\
			echo $$x | sed -ne 's,$(rinc)/\(.*\).h,\1,p';\
		done;\
	) | $(uniq) >$@

# in case a component does not specify diversity,
# just create an empty one
%.diversity:
	$(inform)
	$e$(touch) $@ 

%.div: $(DIVERSITY) %.diversity
	$(inform)
	$e(\
	  if test -s "$(filter %.diversity,$^)"; then\
	  	$(echo) "# Locally-set parameters take precedence";\
	  	$(cat) $(filter %.diversity,$^);\
	  fi;\
	  if test -n "$(filter-out %.diversity,$^)"; then\
	  	$(echo) "# Globally-set parameters as a default";\
	  	$(awk) -vMODULE=$* -vFS="=" '/^#/ { next }\
			{ split($$1,path,"[.]");\
			  if (!path[1]) next;\
			  if (MODULE !~ path[1]) next;\
			  print path[2] "=" $$2\
			}' $(filter-out %.diversity,$^);\
	  fi;\
	) > $@


%.E: %.c %.dep.h %.domains
	$(inform)
	$e$(CC) $(LIME_CFLAGS) $(call dom2hdr,$*.dep.h,$*.domains) -c $*.c -E -o $@

# break cyclical dependency of .E by first processing incomplete module
%.E0: %.c %.domains
	$(inform)
	$e$(CC) $(LIME_CFLAGS) $(call dom2hdr,ctype.h,$*.domains) -c $*.c -E -o $@

%.forwards: %.E0
	$(inform)
	$e$(fwdecl) $^ > $@

%.imps: %.forwards %.domains
	$(inform)
	$e(\
	$(call dom2imp,$*.domains);\
	if test -s $*.forwards -a -n "$(types)"; then\
		$(typeq) -vFILE=$*.forwards $(types) $(redir);\
	fi;\
	) | $(uniq) > $@

%.imports: %.imps
	$(inform)
	$e$(call imps2imp,$<) > $@

%.pdepx.h: %.div %.ren %.imports
	$(inform)
	$e(\
	$(call imp2hdrp,$*.imports,TARGET);\
	$(call nvp2cpp.awk,$(filter-out %.imports,$^));\
	) > $@

%.depx.h: %.div %.ren %.imports
	$(inform)
	$e(\
	$(call imp2hdr,$*.imports);\
	$(call nvp2cpp.awk,$(filter-out %.imports,$^));\
	) > $@

%.dep.h: %.div %.imports
	$(inform)
	$e(\
	$(call imp2hdr,$*.imports);\
	$(call nvp2cpp.awk,$(filter-out %.imports,$^));\
	) > $@


%.ren: %.syms
	$(inform)
	$e( $(echo) "# Rename standard local names";\
	  $(awk) -vMODULE=$* '/^[a-z$$A-Z_][a-z$$A-Z_0-9]*$$/ { print $$0 "=" MODULE "_" $$0 }' $(filter %.syms,$^);\
	) > $@

%.mv: %.syms
	$(inform)
	$e$(awk) -vMODULE=$* '/^[a-z$$A-Z_][a-z$$A-Z_0-9]*$$/ { print $$0, MODULE "_" $$0 }' $< > $@

# generate preprocessed source
%.mod.c: %.c %.depx.h %.domains
	$(inform)
	$E$(CC) $(CFLAGS) $(call dom2hdr,$*.depx.h,$*.domains) $(FLAGS) -E $*.c -o $@ 

# simply compile now
%.mod.o: %.mod.c
	$(inform)
	$E$(CC) $(CFLAGS) -o $@ -c $<


# header handling
%.c: %.h
	$(inform)
	$e(\
	$(echo) '#include LIME';\
	$(echo) '#include <ctype.h>';\
	$(echo) '#include "$(filter %.h,$^)"';\
	) > $@

%.types: %.tu %.E
	$(inform)
	$e$(cparse) -vDIR=$(dir $*) -vSRC=$(filter %.E,$^) -vOP="types,typedefs" -vFNAME=$(notdir $(basename $*.c).h) $(filter %.tu,$^) | $(xmlnorm) -vroot="types" > $@ || $(fail)
	$e$(rm) -f $*.c

#GXF
%.module: %.mod %.imports
	$(inform)
	$e$(composer) -vroot="@" $(filter-out %.imports,$^) $(call imp2type,$*.imports) > $@ || $(fail)

%.graph.xml: %.graph
	$(inform)
	$e$(en2xml) $< >$@ || $(fail)

%.xml: %
	$(inform)
	$e$(en2xml) -vroot=$(subst .,,$(suffix $<)) -vuse_xlink=extended -vdo_dtd=1 $< >$@ || $(fail)

%.gxf: $(modules) %.graph.xml
	$(inform)
	$e$(xmlnorm) -vroot=$(subst .,,$(suffix $@)) -vuse_xlink=extended -vdo_dtd=1 $^ > $@ || $(fail)

%.dot: %.gxf
	$(inform)
	$e$(gxf2dot) $^ > $@ || $(fail)


